<?php
namespace Misuzu;

use RuntimeException;

if(!$msz->getAuthInfo()->getPerms('user')->check(Perm::U_WARNINGS_MANAGE))
    Template::throwError(403);

$usersCtx = $msz->getUsersContext();
$warns = $usersCtx->getWarnings();

$filterUser = null;
if(filter_has_var(INPUT_GET, 'u')) {
    $filterUserId = filter_input(INPUT_GET, 'u', FILTER_SANITIZE_NUMBER_INT);
    try {
        $filterUser = $usersCtx->getUserInfo($filterUserId);
    } catch(RuntimeException $ex) {
        Template::throwError(404);
    }
}

$pagination = new Pagination($warns->countWarnings(userInfo: $filterUser), 10);

if(!$pagination->hasValidOffset())
    Template::throwError(404);

$warnList = [];
$warnInfos = $warns->getWarnings(userInfo: $filterUser, pagination: $pagination);

foreach($warnInfos as $warnInfo) {
    $userInfo = $usersCtx->getUserInfo($warnInfo->getUserId());
    $userColour = $usersCtx->getUserColour($userInfo);

    if(!$warnInfo->hasModId()) {
        $modInfo = null;
        $modColour = null;
    } else {
        $modInfo = $usersCtx->getUserInfo($warnInfo->getModId());
        $modColour = $usersCtx->getUserColour($modInfo);
    }

    $warnList[] = [
        'info' => $warnInfo,
        'user' => $userInfo,
        'user_colour' => $userColour,
        'mod' => $modInfo,
        'mod_colour' => $modColour,
    ];
}

Template::render('manage.users.warnings', [
    'manage_warns' => $warnList,
    'manage_warns_pagination' => $pagination,
    'manage_warns_filter_user' => $filterUser,
]);
